import { createRouter, RouteRecordRaw, createWebHistory } from 'vue-router';

import Layout from '@/layout/MainLayout/Layout.vue';
import ErrorLayout from '@/layout/ErrorLayout/ErrorLayout.vue';

// import { ElIcons } from '@/typings'

export type RouteMeta = {
  /**
   * 页面标题
   */
  title?: string;
  /**
   * menus菜单图标
   */
  icon?: string;
  /**
   * 是否固定标签页
   */
  affix?: boolean;
  /**
   * 是否在面包屑中显示
   */
  breadcrumb?: boolean;
  /**
   * 是否缓存页面
   */
  noCache?: boolean;
  /**
   * 菜单排序显示等级
   */
  rank?: number;

  moduleId?: string;

  /**
   * 是否是根Menu
   */
  rootMenu?: boolean;
};

export type Route = RouteRecordRaw & {
  /**
   * 是否在menu显示
   */
  hidden?: boolean;
  alwaysShow?: boolean;
  /**
   * 设置为noRedirect将不会在面包屑中重定向
   */
  redirect?: string;
  meta?: RouteMeta;
  children?: Route[];
};

export const constantRoutes: Route[] = [
  {
    path: '/redirect',
    component: Layout,
    hidden: true,
    children: [
      {
        path: '/redirect/:path(.*)',
        component: () => import('@/views/Redirect/Redirect.vue'),
        meta: {
          noCache: true,
        },
      },
    ],
  },
  {
    path: '/',
    component: Layout,
    redirect: '/home',
    alwaysShow: true,
    children: [
      {
        path: 'home',
        name: 'Home',
        hidden: false,
        component: () => import('@/views/Home/Home.vue'),
        meta: {
          rank: 10001,
          title: '工作台',
          affix: true,
          rootMenu: true,
        },
      },
    ],
    meta: {
      rank: 1001,
      icon: 'home',
      title: '首页',
      rootMenu: true,
    },
  },
  {
    path: '/EnterpriseInfo',
    redirect: 'noRedirect',
    component: Layout,
    hidden: false,
    alwaysShow: true,
    meta: {
      rank: 10010,
      title: '企业信息',
      rootMenu: true,
      icon: 'home',
    },
    children: [
      {
        path: '/EnterpriseInfoList',
        name: 'EnterpriseInfoList',
        hidden: false,
        alwaysShow: true,
        component: () => import('@/views/EnterpriseInfo/EnterpriseInfo.vue'),
        meta: {
          rank: 10011,
          title: '企业信息',
          // rootMenu: true,
          icon: 'home',
        },
      },
      {
        path: '/EnterpriseInfoDetail/:id',
        name: 'EnterpriseInfoDetail',
        hidden: true,
        component: () => import('@/views/EnterpriseInfo/EnterpriseInfoDetail.vue'),
        meta: {
          rank: 10012,
          title: '企业信息详情',
          // rootMenu: true,
          icon: 'home',
        },
      },
    ],
  },
  {
    path: '/MaterialReview',
    redirect: 'noRedirect',
    component: Layout,
    hidden: false,
    alwaysShow: true,
    meta: {
      rank: 10020,
      title: '材料审核',
      rootMenu: true,
      icon: 'home',
    },
    children: [
      {
        path: '/MaterialReviewList',
        name: 'MaterialReviewList',
        hidden: false,
        alwaysShow: true,
        component: () => import('@/views/MaterialReview/MaterialReview.vue'),
        meta: {
          rank: 10021,
          title: '材料审核',
          // rootMenu: true,
          icon: 'home',
        },
      },
      {
        path: '/MaterialReviewDetail/:id',
        name: 'MaterialReviewDetail',
        hidden: true,
        component: () => import('@/views/MaterialReview/MaterialReviewDetail.vue'),
        meta: {
          rank: 10022,
          title: '材料审核详情',
          rootMenu: false,
        },
      },
      {
        path: '/MaterialReviewAudit/:id',
        name: 'MaterialReviewAudit',
        hidden: true,
        component: () => import('@/views/MaterialReview/MaterialReviewAudit.vue'),
        meta: {
          rank: 10023,
          title: '审核',
          rootMenu: false,
        },
      },
    ],
  },
  {
    path: '/Reward',
    redirect: 'noRedirect',
    component: Layout,
    hidden: false,
    alwaysShow: true,
    meta: {
      rank: 10030,
      title: '平台奖励',
      rootMenu: true,
      icon: 'home',
    },
    children: [
      {
        path: '/RewardGrant',
        name: 'RewardGrant',
        hidden: false,
        alwaysShow: true,
        component: () => import('@/views/Reward/RewardGrant.vue'),
        meta: {
          rank: 10031,
          title: '奖励发放',
          // rootMenu: true,
          icon: 'home',
        },
      },
      {
        path: '/RewardDeclareDetail/:id',
        name: 'RewardDeclareDetail',
        hidden: true,
        component: () => import('@/views/Reward/RewardDeclareDetail.vue'),
        meta: {
          rank: 10032,
          title: '申报详情',
          rootMenu: false,
        },
      },
    ],
  },
  // {
  //   path: '/Syatem',
  //   redirect: 'noRedirect',
  //   component: Layout,
  //   hidden: true,
  //   alwaysShow: false,
  //   meta: {
  //     rank: 10030,
  //     title: '系统配置',
  //     rootMenu: true,
  //     icon: 'home',
  //   },
  //   children: [
  //     {
  //       path: '/Account',
  //       name: 'Account',
  //       hidden: true,
  //       alwaysShow: true,
  //       component: () => import('@/views/Account/AccountManageList.vue'),
  //       meta: {
  //         rank: 10031,
  //         title: '账号管理',
  //         // rootMenu: true,
  //         icon: 'home',
  //       },
  //     },
  //   ],
  // },

  {
    path: '/Login',
    name: 'Login',
    hidden: true,
    component: () => import('@/views/Login/Login.vue'),
    meta: {
      title: '登录',
      rank: 101,
    },
  },
  {
    path: '/Error',
    component: ErrorLayout,
    redirect: '/Error/401',
    hidden: true,
    meta: {
      title: '错误页',
    },
    children: [
      {
        path: '/401',
        name: '401',
        component: () => import('@/views/Error/401.vue'),
        meta: {
          title: '401',
        },
      },
      {
        path: '/404',
        name: '404',
        component: () => import('@/views/Error/404.vue'),
        meta: {
          title: '404',
        },
      },
    ],
  },
];

export const asyncRoutes: Route[] = [
  {
    path: '/System',
    name: 'System',
    redirect: 'noRedirect',
    alwaysShow: true,
    meta: {
      title: '系统配置管理',
      rootMenu: true,
    },
    children: [
      {
        path: '/System/ModuleManage',
        name: 'ModuleManage',
        component: null,
        meta: {
          title: '功能模块管理',
        },
      },
    ],
  },
];

const createRouterFactory = () =>
  createRouter({
    history: createWebHistory(), // hash模式：createWebHashHistory，history模式：createWebHistory
    scrollBehavior(to, from, savedPosition) {
      return new Promise((resolve) => {
        if (savedPosition) {
          return savedPosition;
        } else {
          if (from.meta.saveSrollTop) {
            const top: number = document.documentElement.scrollTop || document.body.scrollTop;
            resolve({ left: 0, top });
          }
        }
      });
    },
    routes: constantRoutes,
  });

export const router = createRouterFactory();

export function resetRouter() {
  const newRouter = createRouterFactory();
  //@ts-ignore
  router.matcher = newRouter.matcher; // reset router
}

export function redirect404() {
  router.replace({ path: '/404' });
}
